@@ -5,7 +5,7 @@ use base 'Class::Accessor::Fast';
 
 __PACKAGE__->mk_accessors(
     qw/action address arguments body base cookies headers hostname match
-      method parameters path snippets uploads/
+      method parameters path protocol secure snippets uploads user/
 );
 
 *args   = \&arguments;
@@ -19,23 +19,6 @@ sub header           { shift->headers->header(@_)           }
 sub referer          { shift->headers->referer(@_)          }
 sub user_agent       { shift->headers->user_agent(@_)       }
 
-sub _assign_values {
-    my ( $self, $map, $values ) = @_;
-
-    while ( my ( $name, $value ) = splice( @{$values}, 0, 2 ) ) {
-
-        if ( exists $map->{$name} ) {
-            for ( $map->{$name} ) {
-                $_ = [$_] unless ref($_) eq "ARRAY";
-                push( @$_, $value );
-            }
-        }
-        else {
-            $map->{$name} = $value;
-        }
-    }
-}
-
 =head1 NAME
 
 Catalyst::Request - Catalyst Request Class
@@ -64,11 +47,15 @@ Catalyst::Request - Catalyst Request Class
     $req->params;
     $req->parameters;
     $req->path;
+    $req->protocol;
     $req->referer;
+    $req->secure;
     $req->snippets;
     $req->upload;
     $req->uploads;
-    $req->user_agent
+    $req->uri;
+    $req->user;
+    $req->user_agent;
 
 See also L<Catalyst>.
 
@@ -184,21 +171,40 @@ sub param {
         return keys %{ $self->parameters };
     }
 
-    my $param = shift;
+    if ( @_ == 1 ) {
 
-    unless ( exists $self->parameters->{$param} ) {
-        return wantarray ? () : undef;
+        my $param = shift;
+
+        unless ( exists $self->parameters->{$param} ) {
+            return wantarray ? () : undef;
+        }
+
+        if ( ref $self->parameters->{$param} eq 'ARRAY' ) {
+            return (wantarray)
+              ? @{ $self->parameters->{$param} }
+              : $self->parameters->{$param}->[0];
+        }
+        else {
+            return (wantarray)
+              ? ( $self->parameters->{$param} )
+              : $self->parameters->{$param};
+        }
     }
 
-    if ( ref $self->parameters->{$param} eq 'ARRAY' ) {
-        return (wantarray)
-          ? @{ $self->parameters->{$param} }
-          : $self->parameters->{$param}->[0];
-    }
-    else {
-        return (wantarray)
-          ? ( $self->parameters->{$param} )
-          : $self->parameters->{$param};
+    if ( @_ > 1 ) {
+
+        while ( my ( $field, $value ) = splice( @_, 0, 2 ) ) {
+
+            if ( exists $self->parameters->{$field} ) {
+                for ( $self->parameters->{$field} ) {
+                    $_ = [$_] unless ref($_) eq "ARRAY";
+                    push( @$_, $value );
+                }
+            }
+            else {
+                $self->parameters->{$field} = $value;
+            }
+        }
     }
 }
 
@@ -220,10 +226,18 @@ Contains the path.
 
     print $c->request->path;
 
+=item $req->protocol
+
+Contains the protocol.
+
 =item $req->referer
 
 Shortcut to $req->headers->referer. Referring page.
 
+=item $req->secure
+
+Contains a boolean whether the communciation is secure.
+
 =item $req->snippets
 
 Returns a reference to an array containing regex snippets.
@@ -237,7 +251,7 @@ A convenient method to $req->uploads.
     $upload  = $c->request->upload('field');
     @uploads = $c->request->upload('field');
     @fields  = $c->request->upload;
-    
+
     for my $upload ( $c->request->upload('field') ) {
         print $upload->filename;
     }
@@ -251,32 +265,67 @@ sub upload {
         return keys %{ $self->uploads };
     }
 
-    my $upload = shift;
+    if ( @_ == 1 ) {
 
-    unless ( exists $self->uploads->{$upload} ) {
-        return wantarray ? () : undef;
+        my $upload = shift;
+
+        unless ( exists $self->uploads->{$upload} ) {
+            return wantarray ? () : undef;
+        }
+
+        if ( ref $self->uploads->{$upload} eq 'ARRAY' ) {
+            return (wantarray)
+              ? @{ $self->uploads->{$upload} }
+              : $self->uploads->{$upload}->[0];
+        }
+        else {
+            return (wantarray)
+               ? ( $self->uploads->{$upload} )
+               : $self->uploads->{$upload};
+        }
     }
 
-    if ( ref $self->uploads->{$upload} eq 'ARRAY' ) {
-        return (wantarray)
-          ? @{ $self->uploads->{$upload} }
-          : $self->uploads->{$upload}->[0];
-    }
-    else {
-        return (wantarray)
-          ? ( $self->uploads->{$upload} )
-          : $self->uploads->{$upload};
+    if ( @_ > 1 ) {
+
+        while ( my ( $field, $upload ) = splice( @_, 0, 2 ) ) {
+
+            if ( exists $self->uploads->{$field} ) {
+                for ( $self->uploads->{$field} ) {
+                    $_ = [$_] unless ref($_) eq "ARRAY";
+                    push( @$_, $upload );
+                }
+            }
+            else {
+                $self->uploads->{$field} = $upload;
+            }
+        }
     }
 }
 
 =item $req->uploads
 
-Returns a reference to a hash containing uploads. Values can be either a 
+Returns a reference to a hash containing uploads. Values can be either a
 hashref or a arrayref containing C<Catalyst::Request::Upload> objects.
 
     my $upload = $c->request->uploads->{field};
     my $upload = $c->request->uploads->{field}->[0];
 
+=item $req->uri
+
+Shortcut for C<< $req->base . $req->path >>.
+
+=cut
+
+sub uri {
+    my $self = shift;
+    my $path = shift || $self->path || '';
+    return $self->base . $path;
+}
+
+=item $req->user
+
+Contains the user name of user if authentication check was successful.
+
 =item $req->user_agent
 
 Shortcut to $req->headers->user_agent. User Agent version string.
